package substitution;

import java.util.HashMap;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.svvrl.goal.core.FinishedException;
import org.svvrl.goal.core.aut.AlphabetType;
import org.svvrl.goal.core.aut.Automaton;
import org.svvrl.goal.core.aut.OmegaUtil;
import org.svvrl.goal.core.aut.Position;
import org.svvrl.goal.core.aut.State;
import org.svvrl.goal.core.aut.fsa.FSA;
import org.svvrl.goal.core.util.FilterRule;
import org.svvrl.goal.gui.ProgressDialog;
import org.svvrl.goal.gui.Tab;
import org.svvrl.goal.gui.UIUtil;
import org.svvrl.goal.gui.Window;
import org.svvrl.goal.gui.action.AutomatonOperationAction;
import org.svvrl.goal.gui.action.ExecutionException;

public class SubstitutionAction extends AutomatonOperationAction<FSA, FSA> {

	private static final long serialVersionUID = 609781865688630675L;
	
	private FSA scope = null;
	private int state;
	private FSA pattern =null;
	
	public SubstitutionAction(Window win) {
		super(win, "Substitution");
	}

	@Override
	public String getToolTip() {
		return "Pattern/Scope Substitution.";
	}

	@Override
	public boolean isApplicable(Automaton aut) {
		return (SubstitutionAction.this.isCompatible(aut));
	}
	
	private boolean isCompatible(Automaton aut){
		return (OmegaUtil.isNBW(aut) &&
				(aut.getAlphabetType() == AlphabetType.PROPOSITIONAL) && 
				(aut.getLabelPosition() == Position.OnTransition));
	}
	
	@Override
	public void preProcess(ProgressDialog dialog) throws Exception {
		super.preProcess(dialog);
		dialog.appendStageMessage("Selecting another automaton.\n");
		
		// GET SCOPE
		this.scope = (FSA)getInput();
		
		// GET substitution state;
		HashMap<String, Integer> states = new HashMap<String, Integer>();
		for (State s : this.scope.getStates()) states.put(s.getIdentityName(), new Integer(s.getID()));		
        if(states.size() == 0)
            throw new ExecutionException("There is no state to be substitued.");
        
        Object obj = JOptionPane.showInputDialog(getWindow(), "Select the state to be substitued.", "the substitution state", -1, null, states.keySet().toArray(), states.keySet().toArray()[0]);
        if(obj == null) throw new FinishedException();
        
        this.state = states.get(obj).intValue();
        
		// GET PATTERN
		FilterRule rule = new FilterRule()	{			
			@Override
			public boolean isSatisfied(Object obj) {
				if (obj instanceof FSA) {
					FSA aut = (FSA)obj;

					return ((aut != scope) && isCompatible(aut));
				}
				return false;
			}			
		};
		
		Tab tab = UIUtil.promptSelection(getWindow().getActiveTab(), "Substitution", 
						"Please select an automaton below to make the substitution of the selected pattern \n into the current scope automaton.", rule);
		if (tab == null) throw new FinishedException();
		
		this.pattern = (FSA)tab.getObject();
		
	}

	@Override
	public FSA execute(ProgressDialog dialog) throws ExecutionException, FinishedException {
		return new SubstitutionConstruction(scope, pattern, state).substitute();
	}

}
